package com.cpp.web.domain.station;

import com.baomidou.mybatisplus.annotation.TableName;

import com.cpp.web.domain.BaseCppEntity;
import com.fasterxml.jackson.annotation.JsonFormat;
import io.swagger.annotations.ApiModel;
import io.swagger.annotations.ApiModelProperty;
import lombok.Data;
import lombok.EqualsAndHashCode;

import javax.validation.constraints.Digits;
import java.math.BigDecimal;
import java.util.Date;

/**
 * 站端nwp
 *
 * @author tl
 * @date 2024-09-23 15:28:33
 */
@Data
@TableName("cpp_nwp_station")
@EqualsAndHashCode(callSuper = true)
@ApiModel(value = "cpp_nwp_station")
public class NwpStation extends BaseCppEntity {


    /**
     * 提前多久预测
     */
    @ApiModelProperty(value = "提前多久预测")
    private Integer forecastHowLongAgo;


    /**
     * 预测时间
     */
    @ApiModelProperty(value = "预测值时间")
    @JsonFormat(pattern = "yyyy-MM-dd HH:mm:ss",timezone="GMT+8")
    private Date time;

    /**
     * 生成日期
     */
    @ApiModelProperty(value = "生成日期")
    @JsonFormat(pattern = "yyyy-MM-dd HH:mm:ss",timezone="GMT+8")
    private Date genDate;

    /**
     * 温度
     */
    @ApiModelProperty(value = "温度")
    @Digits(integer = 10, fraction = 2)
    private BigDecimal t = new BigDecimal(-99);


    /**
     * 湿度
     */
    @ApiModelProperty(value = "湿度")
    @Digits(integer = 10, fraction = 2)
    private BigDecimal rh = new BigDecimal(-99);


    /**
     * 气压
     */
    @ApiModelProperty(value = "气压")
    @Digits(integer = 10, fraction = 2)
    private BigDecimal pressure = new BigDecimal(-99);


    /**
     * 总辐射
     */
    @ApiModelProperty(value = "总辐射")
    @Digits(integer = 10, fraction = 2)
    private BigDecimal globalRadiation = new BigDecimal(-99);


    /**
     * 直辐射
     */
    @ApiModelProperty(value = "直辐射")
    @Digits(integer = 10, fraction = 2)
    private BigDecimal directRadiation = new BigDecimal(-99);


    /**
     * 散辐射
     */
    @ApiModelProperty(value = "散辐射")
    @Digits(integer = 10, fraction = 2)
    private BigDecimal diffuseRadiation = new BigDecimal(-99);


    /**
     * 10米风速
     */
    @ApiModelProperty(value = "10米风速")
    @Digits(integer = 10, fraction = 2)
    private BigDecimal ws10 = new BigDecimal(-99);


    /**
     * 30米风速
     */
    @ApiModelProperty(value = "30米风速")
    @Digits(integer = 10, fraction = 2)
    private BigDecimal ws30 = new BigDecimal(-99);


    /**
     * 50米风速
     */
    @ApiModelProperty(value = "50米风速")
    @Digits(integer = 10, fraction = 2)
    private BigDecimal ws50 = new BigDecimal(-99);


    /**
     * 70米风速
     */
    @ApiModelProperty(value = "70米风速")
    @Digits(integer = 10, fraction = 2)
    private BigDecimal ws70 = new BigDecimal(-99);


    /**
     * 80米风速
     */
    @ApiModelProperty(value = "80米风速")
    @Digits(integer = 10, fraction = 2)
    private BigDecimal ws80 = new BigDecimal(-99);


    /**
     * 90米风速
     */
    @ApiModelProperty(value = "90米风速")
    @Digits(integer = 10, fraction = 2)
    private BigDecimal ws90 = new BigDecimal(-99);


    /**
     * 100米风速
     */
    @ApiModelProperty(value = "100米风速")
    @Digits(integer = 10, fraction = 2)
    private BigDecimal ws100 = new BigDecimal(-99);


    /**
     * 10米风向
     */
    @ApiModelProperty(value = "10米风向")
    @Digits(integer = 10, fraction = 2)
    private BigDecimal wd10 = new BigDecimal(-99);


    /**
     * 30米风向
     */
    @ApiModelProperty(value = "30米风向")
    @Digits(integer = 10, fraction = 2)
    private BigDecimal wd30 = new BigDecimal(-99);


    /**
     * 50米风向
     */
    @ApiModelProperty(value = "50米风向")
    @Digits(integer = 10, fraction = 2)
    private BigDecimal wd50 = new BigDecimal(-99);


    /**
     * 70米风向
     */
    @ApiModelProperty(value = "70米风向")
    @Digits(integer = 10, fraction = 2)
    private BigDecimal wd70 = new BigDecimal(-99);


    /**
     * 80米风向
     */
    @ApiModelProperty(value = "80米风向")
    @Digits(integer = 10, fraction = 2)
    private BigDecimal wd80 = new BigDecimal(-99);


    /**
     * 90米风向
     */
    @ApiModelProperty(value = "90米风向")
    @Digits(integer = 10, fraction = 2)
    private BigDecimal wd90 = new BigDecimal(-99);


    /**
     * 100米风向
     */
    @ApiModelProperty(value = "100米风向")
    @Digits(integer = 10, fraction = 2)
    private BigDecimal wd100 = new BigDecimal(-99);

    /**
     * 轮毂高度风速
     */
    @ApiModelProperty(value = "轮毂高度风速")
    @Digits(integer = 10, fraction = 2)
    private BigDecimal wsHubHeight = new BigDecimal(-99);

    /**
     * 风机轮毂高度风向瞬时值（°）
     */
    @ApiModelProperty(value = "轮毂高度风向（°）")
    @Digits(integer = 10, fraction = 2)
    private BigDecimal wdHubHeight = new BigDecimal(-99);

}
