package com.cpp.web.domain.station;

import com.baomidou.mybatisplus.annotation.TableName;

import com.cpp.web.domain.BaseCppEntity;
import com.fasterxml.jackson.annotation.JsonFormat;
import io.swagger.annotations.ApiModel;
import io.swagger.annotations.ApiModelProperty;
import lombok.Data;
import lombok.EqualsAndHashCode;

import java.math.BigDecimal;
import java.util.Date;

/**
 * cpp_wind_turbine_status_data
 *
 * @author tl
 * @date 2024-09-23 15:28:33
 */
@Data
@TableName("cpp_wind_turbine_status_data")
@EqualsAndHashCode(callSuper = true)
@ApiModel(value = "cpp_wind_turbine_status_data")
public class WindTurbineStatusData extends BaseCppEntity {



    /**
     * 设备主键ID
     */
    @ApiModelProperty(value = "设备主键ID")
    private Long equipmentId;


    /**
     * 状态值为1代表运行，2代表待机，3代表停用，4代表故障
     */
    @ApiModelProperty(value = "状态值为1代表运行，2代表待机，3代表停用，4代表故障")
    private Integer status;


    /**
     * 时间
     */
    @ApiModelProperty(value = "时间")
    @JsonFormat(pattern = "yyyy-MM-dd HH:mm:ss",timezone="GMT+8")
    private Date time;


    /**
     * 有功（KW）
     */
    @ApiModelProperty(value = "有功（KW）")
    private BigDecimal activePower = new BigDecimal(-99);


    /**
     * 无功（KVar）
     */
    @ApiModelProperty(value = "无功（KVar）")
    private BigDecimal reactivePower = new BigDecimal(-99);


    /**
     * 功率因数
     */
    @ApiModelProperty(value = "功率因数")
    private BigDecimal powerFactor = new BigDecimal(-99);


    /**
     * 电压（V）
     */
    @ApiModelProperty(value = "电压（V）")
    private BigDecimal voltage = new BigDecimal(-99);


    /**
     * 电流（A）
     */
    @ApiModelProperty(value = "电流（A）")
    private BigDecimal electricalCurrent = new BigDecimal(-99);


    /**
     * 当日发电量（kW·h）
     */
    @ApiModelProperty(value = "当日发电量（kW·h）")
    private BigDecimal dayElectricQuantity = new BigDecimal(-99);


    /**
     * 当日并网小时
     */
    @ApiModelProperty(value = "当日并网小时")
    private BigDecimal dayGridConnectedHours = new BigDecimal(-99);


    /**
     * 转速（rpm）
     */
    @ApiModelProperty(value = "转速（rpm）")
    private BigDecimal windWheelRatedSpeed = new BigDecimal(-99);


    /**
     * 风速（m/s）
     */
    @ApiModelProperty(value = "风速（m/s）")
    private BigDecimal ws = new BigDecimal(-99);


    /**
     * 风向（°）
     */
    @ApiModelProperty(value = "风向（°）")
    private BigDecimal wd = new BigDecimal(-99);


    /**
     * 温度（℃）
     */
    @ApiModelProperty(value = "温度（℃）")
    private BigDecimal t = new BigDecimal(-99);


    /**
     * 桨距角（°）
     */
    @ApiModelProperty(value = "桨距角（°）")
    private BigDecimal pitchAngle = new BigDecimal(-99);


    /**
     * 累积发电量（MW·h）
     */
    @ApiModelProperty(value = "累积发电量（MW·h）")
    private BigDecimal cumulativeGeneratedEnergy = new BigDecimal(-99);


    /**
     * 是否样板机
     */
    @ApiModelProperty(value = "是否样板机:1是，0不是")
    private Integer isSample;
}
