package com.cpp.framework.web.service;

import java.util.Iterator;
import java.util.concurrent.TimeUnit;

import com.cpp.common.core.cache.LocalCache;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.security.core.Authentication;
import org.springframework.stereotype.Component;
import com.cpp.common.constant.CacheConstants;
import com.cpp.common.core.domain.entity.SysUser;
//import com.cpp.common.core.redis.RedisCache;
import com.cpp.common.exception.user.UserPasswordNotMatchException;
import com.cpp.common.exception.user.UserPasswordRetryLimitExceedException;
import com.cpp.common.utils.SecurityUtils;
import com.cpp.framework.security.context.AuthenticationContextHolder;

/**
 * 登录密码方法
 *
 * @author ruoyi
 */
@Component
public class SysPasswordService
{
//    @Autowired
//    private RedisCache redisCache;

    @Value(value = "${user.password.maxRetryCount}")
    private int maxRetryCount;

    @Value(value = "${user.password.lockTime}")
    private int lockTime;

    /**
     * 登录账户密码错误次数缓存键名
     *
     * @param username 用户名
     * @return 缓存键key
     */
    private String getCacheKey(String username)
    {
        return CacheConstants.PWD_ERR_CNT_KEY + username;
    }

    public void validate(SysUser user)
    {
        Authentication usernamePasswordAuthenticationToken = AuthenticationContextHolder.getContext();
        String username = usernamePasswordAuthenticationToken.getName();
        String password = usernamePasswordAuthenticationToken.getCredentials().toString();
        Integer retryCount = null;
        if (LocalCache.get(getCacheKey(username))!=null){
            retryCount = Integer.parseInt(LocalCache.get(getCacheKey(username)).toString());
        }

        if (retryCount == null)
        {
            retryCount = 0;
        }

        if (retryCount >= Integer.valueOf(maxRetryCount).intValue())
        {
            throw new UserPasswordRetryLimitExceedException(maxRetryCount, lockTime);
        }

        if (!matches(user, password))
        {
            retryCount = retryCount + 1;
            LocalCache.set(getCacheKey(username), retryCount, lockTime * 1000 * 60);
            throw new UserPasswordNotMatchException();
        }
        else
        {
            clearLoginRecordCache(username);
        }
    }

    public boolean matches(SysUser user, String rawPassword)
    {
        return SecurityUtils.matchesPassword(rawPassword, user.getPassword());
    }

    public void clearLoginRecordCache(String loginName)
    {
//        if (redisCache.hasKey(getCacheKey(loginName)))
//        {
//            redisCache.deleteObject(getCacheKey(loginName));
//        }

        Iterator<String> iterator = LocalCache.keySet().iterator();
        while (iterator.hasNext()){
            String str = iterator.next();
            if (str.contains(getCacheKey(loginName))){
                iterator.remove();
            }
        }
    }
}
